%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main file for Experiment 3 (multiple cells) for the TIE tutorial.
% Version 2.0 -
% Related Reference:
% "Transport-of-intensity equationA tutoral"
% Chao Zuo, Jiaji Li, Jiasong Sun,Yao Fan, Jialin Zhang, Linpeng Lu, Runnan Zhang,
% Bowen Wang, Lei Huang, Qian Chen
% last modified on 02/27/2021
% by Chao Zuo (zuochao@njust.edu.cn,surpasszuo@163.com)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Clear everything existing.
clc
close all
clearvars
addpath('.\Functions');
addpath('.\Data\MultipleCell')

%% Setting up system parameters
I0 = double(imread('MultipleCell_I0.tif'))/255;
Iz = double(imread('MultipleCell_Iz.tif'))/255;

% Valid domain within the apecture
Apecture = ones(size(I0));
Apecture(I0<max(max(I0))/10)=0;

Mag = 20;                  % Magnification
Pixelsize = 4.65e-6./Mag;  % Pixelszie
lambda = 550e-9;           % Wavelength (m)
k = 2*pi/lambda;           % Wave number
dz = 2.5e-6;       	       % Defocus distance (m)

% Axial intensity derivative
dIdz = (Iz-I0)/(dz);

% Check if satisfying Energy Conservation
disp(['mean2(Iz-I0)) = ' num2str(mean2(Iz-I0))]);
if abs(mean2(Iz-I0))>max(max(I0))/255    % Need to check the threshold
    warndlg('Conservation of Energy may not be satisfied!');
end

%% Show the images.
figure
subplot(1,3,1)
imshow(I0,[0,1]);
title('In-focus intensity');
subplot(1,3,2)
imshow(Iz,[0,1]);
title('Defocused intensity');
subplot(1,3,3)
imshow(dIdz,[]);
title('Intensity Derivative');


%% Solve TIE with iteration parameters.
RegPara = 1e-9;
IntThr = 0.01;
JudgeFlag_DCT = 1;
JudgeFlag_US = 1;
MaxIterNum = 20;
Method = 'Angular Spectrum';  %'TIE','Angular Spectrum','Fresnel'

%% Solve TIE with FFT-TIE.
Phi_FFT = TIE_FFT_solution(dIdz,I0,Pixelsize,k,RegPara,IntThr);

%% Solve TIE with DCT-TIE.
Phi_DCT = TIE_DCT_solution(dIdz,I0,Pixelsize,k,RegPara,IntThr);

%% Solve TIE with IterDCT-TIE.
Phi_IterDCT = Iter_DCT_solution(dIdz,dz,I0,Pixelsize,k,RegPara,MaxIterNum,JudgeFlag_DCT,Method);

%% Solve TIE with US-TIE.
Phi_US = Universal_Solution(dIdz,dz,I0,Pixelsize,k,RegPara,MaxIterNum,JudgeFlag_US,Method);

%% Show Reconstruction Result
figure;
scrsz = get(0,'ScreenSize');
set(gcf,'Position',scrsz);

subplot(1,4,1)
imshow(Phi_FFT.*Apecture,[]);
colormap(gca,gray);colorbar
title('Reconstructed Phase (FFT-TIE)');
subplot(1,4,2)
imshow(Phi_DCT.*Apecture,[]);
colormap(gca,gray);colorbar
title('Reconstructed Phase (DCT-TIE)');
subplot(1,4,3)
imshow(Phi_IterDCT.*Apecture,[]);
colormap(gca,gray);colorbar
title('Reconstructed Phase (IterDCT-TIE)');
subplot(1,4,4)
imshow(Phi_US.*Apecture,[]);
colormap(gca,gray);colorbar
title('Reconstructed Phase (US-TIE)');
